@@ -134,21 +134,18 @@ setMethod("cellSize", signature(x="SpatRaster"),
 
 
 setMethod ("expanse", "SpatRaster",
-	function(x, unit="m", transform=TRUE) {
-
-		byvalue = FALSE
+	function(x, unit="m", transform=TRUE, byValue=FALSE) {
 		opt <- spatOptions()
-		if (byvalue) {
-			v <- x@ptr$area_by_value(opt)
-			x <- messages(x, "expanse")
-			v <- lapply(1:length(v), function(i) cbind(i, matrix(v[[i]], ncol=2)))
+		v <- x@ptr$sum_area(unit, isTRUE(transform[1]), isTRUE(byValue[1]), opt)
+		x <- messages(x, "expanse")
+		if (byValue) {
+			v <- lapply(1:length(v), function(i) cbind(i, matrix(v[[i]], ncol=2, byrow=TRUE)))
 			v <- do.call(rbind, v)
 			colnames(v) <- c("layer", "value", "area")
 		} else {
-			v <- x@ptr$sum_area(unit, transform, opt)
-			x <- messages(x, "expanse")
+			v <- v[[1]]
 		}
-		return(v)
+		v
 	}
 )
 
