@@ -4,7 +4,7 @@
 # License GPL v3
 
 
-setMethod("init", signature(x="SpatRaster"), 
+setMethod("init", signature(x="SpatRaster"),
 	function(x, fun, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		x <- rast(x)
@@ -24,7 +24,7 @@ setMethod("init", signature(x="SpatRaster"),
 			x@ptr <- x@ptr$initv(fun, opt)
 			messages(x, "init")
 		} else {
-			nc <- ncol(x)
+			nc <- ncol(x) * nlyr(x)
 			b <- writeStart(x, filename, ...)
 			for (i in 1:b$n) {
 				n <- b$nrows[i] * nc;
