@@ -4,8 +4,8 @@
 # License GPL v3
 
 
-setMethod("names", signature(x="SpatRaster"), 
-	function(x) { 
+setMethod("names", signature(x="SpatRaster"),
+	function(x) {
 		nms <- x@ptr$names
 		Encoding(nms) <- "UTF-8"
 		nms
@@ -13,12 +13,12 @@ setMethod("names", signature(x="SpatRaster"),
 )
 
 
-setMethod("names<-", signature(x="SpatRaster"), 
+setMethod("names<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- enc2utf8(as.character(value))
 		if (length(value) != nlyr(x)) {
 			error("names<-", "incorrect number of names")
-		}	
+		}
 		x@ptr <- x@ptr$deepcopy()
 		if (! x@ptr$setNames(value, FALSE)) {
 			error("names<-", "cannot set these names")
@@ -37,7 +37,7 @@ setMethod("names<-", signature(x="SpatRaster"),
 	}
 	if (length(value) != nlyr(x)) {
 		error("names<-", "incorrect number of names")
-	}		
+	}
 	if (validate) {
 		value <- make.names(value, unique = TRUE)
 	}
@@ -53,14 +53,14 @@ setMethod("names<-", signature(x="SpatRaster"),
 	}
 	if (length(value) != ncol(x)) {
 		error("names<-", "incorrect number of names")
-	}		
+	}
 	if (validate) {
 		value <- make.names(value, unique = TRUE)
 	}
 	value
 }
 
-setMethod("set.names", signature(x="SpatRaster"), 
+setMethod("set.names", signature(x="SpatRaster"),
 	function(x, value, index=1:nlyr(x), validate=FALSE)  {
 		value <- .raster_names_check(x, value, index, validate)
 		if (! x@ptr$setNames(value, FALSE)) {
@@ -71,33 +71,7 @@ setMethod("set.names", signature(x="SpatRaster"),
 )
 
 
-setMethod("names", signature(x="SpatRasterDataset"), 
-	function(x) { 
-		nms <- x@ptr$names
-		Encoding(nms) <- "UTF-8"
-		nms
-	}
-)
-
-
-setMethod("names<-", signature(x="SpatRasterDataset"), 
-	function(x, value) {
-		x@ptr <- x@ptr$deepcopy()
-		x@ptr$names <- enc2utf8(as.character(value))
-		x
-	}
-)
-
-setMethod("set.names", signature(x="SpatRasterDataset"), 
-	function(x, value, index=1:length(x), validate=FALSE)  {
-		value <- .raster_names_check(x, value, index, validate)
-		x@ptr$names <- value
-		invisible(TRUE)
-	}
-)
-
-
-setMethod("varnames", signature(x="SpatRasterDataset"), 
+setMethod("names", signature(x="SpatRasterDataset"),
 	function(x) {
 		nms <- x@ptr$names
 		Encoding(nms) <- "UTF-8"
@@ -106,7 +80,33 @@ setMethod("varnames", signature(x="SpatRasterDataset"),
 )
 
 
-setMethod("varnames<-", signature(x="SpatRasterDataset"), 
+setMethod("names<-", signature(x="SpatRasterDataset"),
+	function(x, value) {
+		x@ptr <- x@ptr$deepcopy()
+		x@ptr$names <- enc2utf8(as.character(value))
+		x
+	}
+)
+
+setMethod("set.names", signature(x="SpatRasterDataset"),
+	function(x, value, index=1:length(x), validate=FALSE)  {
+		value <- .raster_names_check(x, value, index, validate)
+		x@ptr$names <- value
+		invisible(TRUE)
+	}
+)
+
+
+setMethod("varnames", signature(x="SpatRasterDataset"),
+	function(x) {
+		nms <- x@ptr$names
+		Encoding(nms) <- "UTF-8"
+		nms
+	}
+)
+
+
+setMethod("varnames<-", signature(x="SpatRasterDataset"),
 	function(x, value) {
 		value <- enc2utf8(as.character(value))
 		x@ptr <- x@ptr$deepcopy()
@@ -117,23 +117,23 @@ setMethod("varnames<-", signature(x="SpatRasterDataset"),
 
 
 
-setMethod("names", signature(x="SpatVector"), 
-	function(x) { 
+setMethod("names", signature(x="SpatVector"),
+	function(x) {
 		nms <- x@ptr$names
 		Encoding(nms) <- "UTF-8"
 		nms
 	}
 )
 
-setMethod("names", signature(x="SpatVectorProxy"), 
-	function(x) { 
+setMethod("names", signature(x="SpatVectorProxy"),
+	function(x) {
 		nms <- x@ptr$v$names
 		Encoding(nms) <- "UTF-8"
 		nms
 	}
 )
 
-setMethod("names<-", signature(x="SpatVector"), 
+setMethod("names<-", signature(x="SpatVector"),
 	function(x, value)  {
 		if (length(value) != ncol(x)) {
 			error("names<-,SpatVector", "incorrect number of names")
@@ -148,7 +148,7 @@ setMethod("names<-", signature(x="SpatVector"),
 	}
 )
 
-setMethod("set.names", signature(x="SpatVector"), 
+setMethod("set.names", signature(x="SpatVector"),
 	function(x, value, index=1:ncol(x), validate=FALSE)  {
 		value <- .vector_names_check(x, value, index, validate)
 		x@ptr$names <- value
@@ -156,15 +156,15 @@ setMethod("set.names", signature(x="SpatVector"),
 	}
 )
 
-setMethod("varnames", signature(x="SpatRaster"), 
-	function(x) { 
+setMethod("varnames", signature(x="SpatRaster"),
+	function(x) {
 		nms <- x@ptr$get_sourcenames()
 		Encoding(nms) <- "UTF-8"
 		nms
 	}
 )
 
-setMethod("varnames<-", signature(x="SpatRaster"), 
+setMethod("varnames<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- enc2utf8(as.character(value))
 		x@ptr <- x@ptr$deepcopy()
@@ -176,8 +176,8 @@ setMethod("varnames<-", signature(x="SpatRaster"),
 )
 
 
-setMethod("longnames", signature(x="SpatRasterDataset"), 
-	function(x) { 
+setMethod("longnames", signature(x="SpatRasterDataset"),
+	function(x) {
 		nms <- x@ptr$long_names
 		Encoding(nms) <- "UTF-8"
 		nms
@@ -185,15 +185,15 @@ setMethod("longnames", signature(x="SpatRasterDataset"),
 )
 
 
-setMethod("longnames", signature(x="SpatRaster"), 
-	function(x) { 
+setMethod("longnames", signature(x="SpatRaster"),
+	function(x) {
 		nms <- x@ptr$get_sourcenames_long()
 		Encoding(nms) <- "UTF-8"
 		nms
 	}
 )
 
-setMethod("longnames<-", signature(x="SpatRasterDataset"), 
+setMethod("longnames<-", signature(x="SpatRasterDataset"),
 	function(x, value)  {
 		x@ptr <- x@ptr$deepcopy()
 		x@ptr$long_names <- enc2utf8(as.character(value))
@@ -202,7 +202,7 @@ setMethod("longnames<-", signature(x="SpatRasterDataset"),
 )
 
 
-setMethod("longnames<-", signature(x="SpatRaster"), 
+setMethod("longnames<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		x@ptr <- x@ptr$deepcopy()
 		value <- enc2utf8(as.character(value))
@@ -213,3 +213,20 @@ setMethod("longnames<-", signature(x="SpatRaster"),
 	}
 )
 
+
+setMethod("names", signature(x="SpatVectorCollection"),
+	function(x) {
+		nms <- x@ptr$names
+		Encoding(nms) <- "UTF-8"
+		nms
+	}
+)
+
+
+setMethod("names<-", signature(x="SpatVectorCollection"),
+	function(x, value) {
+		x@ptr <- x@ptr$deepcopy()
+		x@ptr$setNames(enc2utf8(as.character(value)), FALSE)
+		x
+	}
+)
