@@ -1,9 +1,7 @@
 
 
-# not exported
-if (!isGeneric("blockSize")) {setGeneric("blockSize", function(x, ...) standardGeneric("blockSize"))}
-setMethod("blockSize", signature(x="SpatRaster"),
-	function(x, n) {
+setMethod("blocks", signature(x="SpatRaster"),
+	function(x, n=4) {
 		opt <- spatOptions("", FALSE, ncopies=n)
 		b <- x@ptr$getBlockSizeR(n, opt$memfrac)
 		b$row <- b$row + 1
