@@ -32,31 +32,6 @@ double roundn(double x, int n);
 double signif(double x, unsigned n);
 
 
-template <typename Iterator>
-void minmaxlim(Iterator start, Iterator end, double &vmin, double &vmax, const double &lmin, const double &lmax) {
-    vmin = std::numeric_limits<double>::max();
-    vmax = std::numeric_limits<double>::lowest();
-    bool none = true;
-	for (Iterator v = start; v !=end; ++v) {
-		if (!std::isnan(*v)) {
-			if (*v >= lmin && *v <= lmax) {
-				if (*v > vmax) {
-					vmax = *v;
-					none = false;
-				}
-				if (*v < vmin) {
-					vmin = *v;
-				}
-			}
-		}
-    }
-    if (none) {
-        vmin = NAN;
-        vmax = NAN;
-    }
-}
-
-
 template <typename Iterator>
 void minmax(Iterator start, Iterator end, double &vmin, double &vmax) {
     vmin = std::numeric_limits<double>::max();
